{
TimDBImage Component
http://www.marteens.com/ejemplos.htm
imDBJPEG

Este package contiene el cdigo fuente del componente imDBImage,
que permite trabajar con imgenes almacenadas campos BLOB en el formato JPEG.
Acepta tambin imgenes de mapas de bits. Soporta el uso del Portapapeles, comprimiendo la imagen pegada antes de guardarla en la tabla
}

{
TCLDBImage Component
 Christian Loayza
christian.loayza@gmail.com
Version 0.01
Abril 2013

DESCRIPCION: Este componente se basa en la propuesta de TimDBImage, que permite trabajar con imgenes almacenadas campos BLOB
             en el formato ICO, BMP, JPG, PNG, GIF, TIF

MI APORTE
   Incorporacin de nuevos formatos ICO, PNG, GIF, TIF, WMF, EMF disponibles en Delphi XE2 y supongo que en XE...
   Para Delphi 2007, se usa la la unit pngimage.pas que forma parte de los componentes PNG Components.

Nuevas Propiedades:
   ImageType (TImageType)  : Tipo de imagen que contiene el campo blob.
   TitleEmptyImage (String): Titulo a desplegar cuando el campo no contiene una imagen.
}

unit CLDBImage;

interface
{.$DEFINE PNG_Component}
{$DEFINE TIF_Imagen}
{$DEFINE WMF_Imagen}

{$IFDEF VER220} //Delphi Xe
{$DEFINE Delphi_XE}
{$ENDIF}

{$IFDEF VER230} //Delphi Xe2
{$DEFINE Delphi_XE}
{$ENDIF}

uses Windows, Messages, SysUtils, Classes,
     Controls, Graphics, DB, DBCtrls, forms,
{$IFDEF Delphi_XE}  //Delphi Xe..Xe2
     Vcl.Imaging.jpeg, Vcl.Imaging.pngimage, Vcl.Imaging.GIFImg;
{$ELSE}
     jpeg, pngimage, GIFImg;
{$ENDIF}
type
  TImageType=(itUnknown,itICO, itCUR, itBMP,itJPG, itPNG, itGIF {$IFDEF TIF_Imagen}, itTIF{$ENDIF}{$IFDEF WMF_Imagen}, itWMF, itEMF{$ENDIF});
  TImageTypes=set of TImageType;

const
   TImageExt:array[TImageType] Of string=('*.*','.ICO','.CUR','.BMP','.JPG','.PNG','.GIF'{$IFDEF TIF_Imagen},'.TIF'{$ENDIF}{$IFDEF WMF_Imagen},'.WMF','.EMF'{$ENDIF});

type
  TCLDBImage = class(TCustomControl)
  private
    FAutoDisplay: Boolean;
    FBorderStyle: TBorderStyle;
    FCenter: Boolean;
    FQuality: TJPEGQualityRange;
    FDataLink: TFieldDataLink;
    FPicture: TPicture;
    FPictureLoaded: Boolean;
    FProportional: Boolean;
    FQuickDraw: Boolean;
    FStretch: Boolean;
    FImageType:TImageType;
    fImageWidth:Integer;
    fImageHeight:Integer;
    fTitleEmptyImage:string;
    procedure DataChange(Sender: TObject);
    function GetDataField: string;
    function GetDataSource: TDataSource;
    function GetField: TField;
    function GetReadOnly: Boolean;
    procedure PictureChanged(Sender: TObject);
    procedure SetAutoDisplay(Value: Boolean);
    procedure SetBorderStyle(Value: TBorderStyle);
    procedure SetCenter(Value: Boolean);
    procedure SetDataField(const Value: string);
    procedure SetDataSource(Value: TDataSource);
    procedure SetTitleEmptyImage(Value:string);
    procedure SetPicture(Value: TPicture);
    procedure SetProportional(Value: Boolean);
    procedure SetReadOnly(Value: Boolean);
    procedure SetStretch(Value: Boolean);
    procedure UpdateData(Sender: TObject);
    procedure CMEnter(var Message: TCMEnter); message CM_ENTER;
    procedure CMExit(var Message: TCMExit); message CM_EXIT;
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
    procedure WMLButtonDown(var Message: TWMLButtonDown); message WM_LBUTTONDOWN;
    procedure WMLButtonDblClk(var Message: TWMLButtonDblClk); message WM_LBUTTONDBLCLK;
    procedure WMCut(var Message: TMessage); message WM_CUT;
    procedure WMCopy(var Message: TMessage); message WM_COPY;
    procedure WMPaste(var Message: TMessage); message WM_PASTE;
    procedure WMSize(var Message: TMessage); message WM_SIZE;
    procedure CMGetDataLink(var Message: TMessage); message CM_GETDATALINK;
  protected
    procedure LoadPicture;
    procedure AssignFieldToPicture(APicture:TPicture);
    procedure AssignPictureToField;
    procedure LoadImagen(Stream:TStream;APicture:TPicture);
    procedure CreateParams(var Params: TCreateParams); override;
    function DestRect(APicture: TPicture): TRect;
    function GetPalette: HPALETTE; override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: Char); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure CopyToClipboard;
    procedure CutToClipboard;
    procedure LoadFromFile(const Filename: string);

    ///	<summary>
    ///	  Permite guadar el contenido a un archivo externo, verificando la
    ///	  extensin de acuerdo al formato almacenado, realizando el cambio de
    ///	  la extensin cuando sea requerido
    ///	</summary>
    ///	<param name="Filename">
    ///	  Nombre que tomara el archivo de salida
    ///	</param>
    procedure SaveToFile(const Filename: string);
    procedure PasteFromClipboard;
    property Field: TField read GetField;
    property Picture: TPicture read FPicture write SetPicture;
  published
    property Align;
    property Anchors;
    property AutoDisplay: Boolean read FAutoDisplay write SetAutoDisplay default True;
    property BorderStyle: TBorderStyle read FBorderStyle write SetBorderStyle default bsSingle;
    property Center: Boolean read FCenter write SetCenter default True;
    property Color;
    property CompressionQuality: TJPEGQualityRange read FQuality write FQuality default 90;
    property Constraints;
    property Ctl3D;
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property ImageType:TImageType read fImageType write fImageType;
    property ImageWidth:Integer read fImageWidth write fImageWidth;
    property ImageHeight:Integer read fImageHeight write fImageHeight;
    property TitleEmptyImage:string read fTitleEmptyImage write SetTitleEmptyImage;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Font;
    property ParentColor default False;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property Proportional: Boolean read FProportional write SetProportional default False;
    property ReadOnly: Boolean read GetReadOnly write SetReadOnly default False;
    property QuickDraw: Boolean read FQuickDraw write FQuickDraw default True;
    property ShowHint;
    property Stretch: Boolean read FStretch write SetStretch default False;
    property TabOrder;
    property TabStop default True;
    property Visible;
    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
  end;

function TypeImageInStream(Stream:TStream):TImageType;

procedure Register;

implementation

uses Clipbrd, Types;

function TypeImageInStream(Stream:TStream):TImageType;
const
  HeaderJGP: array[0..2] of Byte = ($FF, $D8, $FF);
  HeaderPNG: array[0..2] of Byte = ($89, $50, $4E);
  HeaderGIF: array[0..2] of Byte = ($47, $49, $46);
  HeaderICO: array[0..2] of Byte = ($00, $00, $01);
  HeaderCUR: array[0..2] of Byte = ($00, $00, $02);
  HeaderBMP: array[0..2] of Byte = ($42, $4D, $00);
  HeaderTIF: array[0..2] of Byte = ($49, $49, $2A);
  HeaderWMF: array[0..2] of Byte = ($D7, $CD, $C6);
  HeaderEMF: array[0..2] of Byte = ($01, $00, $00);
var
   OldPos:integer;
   Buf: array[0..2] of Byte;
begin
     Result:=itUnknown;
     OldPos:=Stream.Position;
     FillChar(Buf, SizeOf(Buf), 0);
     try
        Stream.Position:=0;
        Stream.ReadBuffer(Buf[0],SizeOf(Buf));

        if CompareMem(@HeaderICO[0], @Buf[0], SizeOf(Buf)) then
           Result:=itICO
        else if CompareMem(@HeaderCUR[0], @Buf[0], SizeOf(Buf)) then
           Result:=itCUR
        else if CompareMem(@HeaderBMP[0], @Buf[0], SizeOf(Buf)-1) then
           Result:=itBMP
        else if CompareMem(@HeaderJGP[0], @Buf[0], SizeOf(Buf)) then
           Result:=itJPG
        else if CompareMem(@HeaderPNG[0], @Buf[0], SizeOf(Buf)) then
           Result:=itPNG
        else if CompareMem(@HeaderGIF[0], @Buf[0], SizeOf(Buf)) then
           Result:=itGIF
        {$IFDEF TIF_Imagen}
        else if CompareMem(@HeaderTIF[0], @Buf[0], SizeOf(Buf)) then
           Result:=itTIF
        {$ENDIF}
        {$IFDEF WMF_Imagen}
        else if CompareMem(@HeaderWMF[0], @Buf[0], SizeOf(Buf)) then
           Result:=itWMF
        else if CompareMem(@HeaderEMF[0], @Buf[0], SizeOf(Buf)) then
           Result:=itEMF
        {$ENDIF};
     finally
        Stream.Position:=OldPos;
     end;
end;

procedure Register;
begin
     RegisterComponents('Data Controls',[TCLDBImage]);
end;

constructor TCLDBImage.Create(AOwner: TComponent);
begin
     inherited Create(AOwner);
     ControlStyle := ControlStyle + [csOpaque, csReplicatable, csNeedsBorderPaint];
     if not NewStyleControls then ControlStyle := ControlStyle + [csFramed];
     Width := 105;
     Height := 105;
     TabStop := True;
     ParentColor := False;
     FPicture := TPicture.Create;
     FPicture.OnChange := PictureChanged;
     FBorderStyle := bsSingle;
     FAutoDisplay := True;
     FCenter := True;
     FDataLink := TFieldDataLink.Create;
     FDataLink.Control := Self;
     FDataLink.OnDataChange := DataChange;
     FDataLink.OnUpdateData := UpdateData;
     FQuickDraw := True;
     FQuality := 90;
     FImageType:=itUnknown;
     fImageHeight:=0;
     fImageWidth :=0;
     fTitleEmptyImage:='(* Sin Imagen *)';
end;

procedure TCLDBImage.SetTitleEmptyImage(Value:string);
begin
     If value<>fTitleEmptyImage then
     begin
          fTitleEmptyImage:=Value;
          Invalidate;
     end;
end;

destructor TCLDBImage.Destroy;
begin
     FPicture.Free;
     FDataLink.Free;
     FDataLink := nil;
     inherited Destroy;
end;

procedure TCLDBImage.CreateParams(var Params: TCreateParams);
begin
     inherited CreateParams(Params);
     with Params do
     begin
          if FBorderStyle = bsSingle then
            if NewStyleControls and Ctl3D then
               ExStyle := ExStyle or WS_EX_CLIENTEDGE
            else
               Style := Style or WS_BORDER;
          WindowClass.style := WindowClass.style and not (CS_HREDRAW or CS_VREDRAW);
     end;
end;


procedure TCLDBImage.LoadFromFile(const Filename: string);
begin
     FPicture.LoadFromFile(Filename);
end;

procedure TCLDBImage.SaveToFile(const Filename: string);
Var
   AFilename:string;
begin
     if (FImageType<>itUnknown) then
     begin
          AFilename:=Filename;
          if (UpperCase(ExtractFileExt(AFilename))<>TImageExt[FImageType]) then
             AFilename:=ChangeFileExt(AFilename,TImageExt[FImageType]);

          FPicture.SaveToFile(AFilename);
     end;
end;

procedure TCLDBImage.LoadImagen(Stream:TStream;APicture:TPicture);
begin
     fImageType  :=TypeImageInStream(Stream);
     fImageHeight:=0;
     fImageWidth :=0;
     try
        case (fImageType) of
             itICO:APicture.Graphic := TIcon.Create;
             itBMP:APicture.Graphic := TBitmap.Create;
             itJPG:APicture.Graphic := TJPEGImage.Create;
             {$IFDEF PNG_Component}
             itPNG:APicture.Graphic := TPngObject.Create;
             {$ELSE}
             itPNG:APicture.Graphic := TPNGImage.Create;
             {$ENDIF}
             itGIF:APicture.Graphic := TGIFImage.Create;
             {$IFDEF TIF_Imagen}
             itTIF:APicture.Graphic := TWICImage.Create;
             {$ENDIF}
             {$IFDEF WMF_Imagen}
             itWMF:APicture.Graphic := TMetafile.Create;
             itEMF:APicture.Graphic := TMetafile.Create;
             {$ENDIF}
        end;

        If (fImageType<>itUnknown) then
        begin
             APicture.Graphic.LoadFromStream(Stream);
             fImageHeight:=APicture.Graphic.Height;
             fImageWidth :=APicture.Graphic.Width;
        end;

     except
          raise
     end;
end;

procedure TCLDBImage.AssignFieldToPicture(APicture:TPicture);
var
   Stream:TMemoryStream;
   Reload:boolean;
begin
     if not Assigned(FDataLink.Field)  then
     begin
          APicture.Graphic := nil;
          Exit;
     end;

     Reload:=APicture<>nil;
     if not Reload then
        APicture:=Picture;

     if (not FPictureLoaded or Reload) and FDataLink.Field.IsBlob then
     begin
          if (not FDataLink.Field.IsBlob and FDataLink.Field.IsNull)
             or (TBlobField(FDataLink.Field).BlobSize=0) then
          begin
               APicture.Graphic:=nil;
               FImageType      :=itUnknown;
          end
          else
          begin
               Stream:=TMemoryStream.Create;
               try
                  TBlobField(FDataLink.Field).SaveToStream(Stream);
                  Stream.Position:=0;

                  LoadImagen(Stream, APicture);
               finally
                Stream.Free;
               end;
          end;
      end;
end;

procedure TCLDBImage.AssignPictureToField;
var
  Stream: TStream;
begin
     Stream := FDataLink.DataSet.CreateBlobStream(TBlobField(FDataLink.Field), bmWrite);
     try
        if Picture.Graphic is TJPEGImage then
          TJPEGImage(Picture.Graphic).CompressionQuality := FQuality;

        Picture.Graphic.SaveToStream(Stream);

        fImageType:=TypeImageInStream(Stream);

        If (fImageType=itUnknown) then
            FDataLink.Field.Clear;
     finally
        Stream.Free;
     end;
end;

procedure TCLDBImage.DataChange(Sender: TObject);
begin
     Picture.Graphic := nil;
     FPictureLoaded := False;
     if FAutoDisplay then
        LoadPicture;
end;

procedure TCLDBImage.CMGetDataLink(var Message: TMessage);
begin
     Message.Result := Integer(FDataLink);
end;

function TCLDBImage.GetDataField: string;
begin
     Result := FDataLink.FieldName;
end;

function TCLDBImage.GetDataSource: TDataSource;
begin
     Result := FDataLink.DataSource;
end;

function TCLDBImage.GetField: TField;
begin
     Result := FDataLink.Field;
end;

function TCLDBImage.GetPalette: HPALETTE;
begin
     Result := 0;
     if FPicture.Graphic is TBitmap then
       Result := TBitmap(FPicture.Graphic).Palette
     else if FPicture.Graphic is TJPEGImage then
       Result := TJPEGImage(FPicture.Graphic).Palette
     {$IFDEF PNG_Component}
     else if FPicture.Graphic is TPNGObject then
       Result :=TPNGObject(FPicture.Graphic).Palette
     {$ELSE}
     else if FPicture.Graphic is TPNGImage then
       Result := TPNGImage(FPicture.Graphic).Palette
     {$ENDIF}
     else if FPicture.Graphic is TGIFImage then
       Result := TGIFImage(FPicture.Graphic).Palette
     else if FPicture.Graphic is TICON then
       Result := TICON(FPicture.Graphic).Palette
     {$IFDEF TIF_Imagen}
     else if FPicture.Graphic is TWICImage then
       Result := TWICImage(FPicture.Graphic).Palette
     {$ENDIF}
     {$IFDEF WMF_Imagen}
     else if FPicture.Graphic is TMetafile then
       Result := TMetaFile(FPicture.Graphic).Palette
     {$ENDIF};
end;

function TCLDBImage.DestRect(APicture: TPicture): TRect;
var
  w, h, cw, ch: Integer;
  xyaspect: Double;
begin
     w := APicture.Width;
     h := APicture.Height;
     cw := ClientWidth;
     ch := ClientHeight;
     if Stretch or (Proportional and ((w > cw) or (h > ch))) then
     begin
          if Proportional and (w > 0) and (h > 0) then
          begin
               xyaspect := w / h;
               if w > h then
               begin
                    w := cw;
                    h := Trunc(cw / xyaspect);
                    if h > ch then  // woops, too big
                    begin
                         h := ch;
                         w := Trunc(ch * xyaspect);
                    end;
               end
               else
               begin
                    h := ch;
                    w := Trunc(ch * xyaspect);
                    if w > cw then  // woops, too big
                    begin
                      w := cw;
                      h := Trunc(cw / xyaspect);
                    end;
               end;
          end
          else
          begin
               w := cw;
               h := ch;
          end;
     end;

     with Result do
     begin
          Left := 0;
          Top := 0;
          Right := w;
          Bottom := h;
     end;

     if Center then
       OffsetRect(Result, (cw - w) div 2, (ch - h) div 2);
end;

function TCLDBImage.GetReadOnly: Boolean;
begin
     Result := FDataLink.ReadOnly;
end;

procedure TCLDBImage.LoadPicture;
begin
     if not FPictureLoaded and (not Assigned(FDataLink.Field) or FDataLink.Field.IsBlob) then
        AssignFieldToPicture(Picture);
end;

procedure TCLDBImage.Paint;
var
  Size: TSize;
  R: TRect;
  S: string;
  DrawPict: TPicture;
  Form: TCustomForm;
  Pal: HPalette;

  procedure DisplayMessage(Canvas:TCanvas; Text:string);
  begin
       with Canvas do
       begin
            if fTitleEmptyImage<>'' then
               Text:=fTitleEmptyImage;

            Font := Self.Font;
            Font.Style:=[fsBold];
            Size := TextExtent(Text);
            R := ClientRect;
            TextRect(R, (R.Right - Size.cx) div 2, (R.Bottom - Size.cy) div 2, Text);
       end;
  end;
begin
     with Canvas do
     begin
          Brush.Style := bsSolid;
          Brush.Color := Color;
          if FPictureLoaded or (csPaintCopy in ControlState) then
          begin
               DrawPict := TPicture.Create;
               Pal := 0;
               try
                  if (csPaintCopy in ControlState) and
                    Assigned(FDataLink.Field) and FDataLink.Field.IsBlob then
                  begin
                       AssignFieldToPicture(DrawPict);
                       if DrawPict.Graphic is TBitmap then
                         DrawPict.Bitmap.IgnorePalette := QuickDraw;
                  end
                  else
                  begin
                       DrawPict.Assign(Picture);
                       if Focused and (DrawPict.Graphic <> nil) and (DrawPict.Graphic.Palette <> 0) then
                       begin { Control has focus, so realize the bitmap palette in foreground }
                            Pal := SelectPalette(Handle, DrawPict.Graphic.Palette, False);
                            RealizePalette(Handle);
                       end;
                  end;
                  {Si la imagen es transparente se debe borrar el lienzo, de lo contrario
                   quedara parte de la imagen anterior. CLoayza}
                  if (DrawPict.Graphic = nil) or DrawPict.Graphic.Empty or DrawPict.Graphic.Transparent then
                     FillRect(ClientRect);

                  R := DestRect(DrawPict);
                  StretchDraw(R, DrawPict.Graphic);
                  if not (Stretch and not Proportional) then
                  begin
                    ExcludeClipRect(Handle, R.Left, R.Top, R.Right, R.Bottom);
                    FillRect(ClientRect);
                    SelectClipRgn(Handle, 0);
                  end;

               finally
                 {Para desplegar TitleEmptyImage}
                 if DrawPict.Graphic = nil then
                    DisplayMessage(Canvas, '');

                 if Pal <> 0 then
                    SelectPalette(Handle, Pal, True);
                 DrawPict.Free;
               end;
          end
          else
          begin
               Font := Self.Font;
               if FDataLink.Field <> nil then
                  S := FDataLink.Field.DisplayLabel
               else
                  S := Name;
               S := '(' + S + ')';

               DisplayMessage(Canvas, S);
          end;
          Form := GetParentForm(Self);
          if (Form <> nil) and (Form.ActiveControl = Self) and
            not (csDesigning in ComponentState) and
            not (csPaintCopy in ControlState) then
          begin
               Brush.Color := clWindowFrame;
               FrameRect(ClientRect);
          end;
     end;
end;

procedure TCLDBImage.PictureChanged(Sender: TObject);
begin
     if FPictureLoaded then
        FDataLink.Modified;
     FPictureLoaded := True;
     Invalidate;
end;

procedure TCLDBImage.Notification(AComponent: TComponent;Operation: TOperation);
begin
     inherited Notification(AComponent, Operation);
     if (Operation = opRemove) and (FDataLink <> nil) and
        (AComponent = DataSource) then DataSource := nil;
end;

procedure TCLDBImage.SetAutoDisplay(Value: Boolean);
begin
     if FAutoDisplay <> Value then
     begin
          FAutoDisplay := Value;
          if Value then LoadPicture;
     end;
end;

procedure TCLDBImage.SetBorderStyle(Value: TBorderStyle);
begin
     if FBorderStyle <> Value then
     begin
          FBorderStyle := Value;
          RecreateWnd;
     end;
end;

procedure TCLDBImage.SetCenter(Value: Boolean);
begin
     if FCenter <> Value then
     begin
          FCenter := Value;
          Invalidate;
     end;
end;

procedure TCLDBImage.SetDataField(const Value: string);
begin
     FDataLink.FieldName := Value;
end;

procedure TCLDBImage.SetDataSource(Value: TDataSource);
begin
     if not (FDataLink.DataSourceFixed and (csLoading in ComponentState)) then
        FDataLink.DataSource := Value;
     if Value <> nil then Value.FreeNotification(Self);
end;

procedure TCLDBImage.SetPicture(Value: TPicture);
begin
     FPicture.Assign(Value);
end;

procedure TCLDBImage.SetProportional(Value: Boolean);
begin
     if FProportional <> Value then
     begin
          FProportional := Value;
          Invalidate;
     end;
end;

procedure TCLDBImage.SetReadOnly(Value: Boolean);
begin
     FDataLink.ReadOnly := Value;
end;

procedure TCLDBImage.SetStretch(Value: Boolean);
begin
     if FStretch <> Value then
     begin
          FStretch := Value;
          Invalidate;
     end;
end;

procedure TCLDBImage.UpdateData(Sender: TObject);
begin
     AssignPictureToField;
end;

procedure TCLDBImage.CopyToClipboard;
begin
     if Picture.Graphic <> nil then Clipboard.Assign(Picture);
end;

procedure TCLDBImage.CutToClipboard;
begin
     if Picture.Graphic <> nil then
       if FDataLink.Edit then
       begin
            CopyToClipboard;
            Picture.Graphic := nil;
       end;
end;

procedure TCLDBImage.PasteFromClipboard;
var
  B: TBitmap;
begin
     if Clipboard.HasFormat(CF_BITMAP) and FDataLink.Edit then
     begin
          B := TBitmap.Create;
          try
             B.Assign(Clipboard);
             Picture.Graphic := TJPEGImage.Create;
             Picture.Graphic.Assign(B);
             TJPEGImage(Picture.Graphic).Compress;
          finally
             B.Free;
          end;
     end;
end;

procedure TCLDBImage.KeyDown(var Key: Word; Shift: TShiftState);
begin
     inherited KeyDown(Key, Shift);
     case Key of
     VK_INSERT:
         if ssShift in Shift then
            PasteFromClipBoard
         else  if ssCtrl in Shift then
            CopyToClipBoard;
     VK_DELETE:
         if ssShift in Shift then CutToClipBoard;
     end;
end;

procedure TCLDBImage.KeyPress(var Key: Char);
begin
     inherited KeyPress(Key);
     case Key of
       ^X: CutToClipBoard;
       ^C: CopyToClipBoard;
       ^V: PasteFromClipBoard;
       #13: LoadPicture;
       #27: FDataLink.Reset;
     end;
end;

procedure TCLDBImage.CMEnter(var Message: TCMEnter);
begin
     Invalidate; { Draw the focus marker }
     inherited;
end;

procedure TCLDBImage.CMExit(var Message: TCMExit);
begin
     try
        if Assigned(DataSource) and Assigned(DataSource.DataSet) and
          (DataSource.DataSet.State in [dsInsert, dsEdit]) then
          FDataLink.UpdateRecord;
     except
         SetFocus;
         raise;
     end;
     Invalidate; { Erase the focus marker }
     inherited;
end;

procedure TCLDBImage.CMTextChanged(var Message: TMessage);
begin
     inherited;
     if not FPictureLoaded then Invalidate;
end;

procedure TCLDBImage.WMLButtonDown(var Message: TWMLButtonDown);
begin
     if TabStop and CanFocus then SetFocus;
     inherited;
end;

procedure TCLDBImage.WMLButtonDblClk(var Message: TWMLButtonDblClk);
begin
     LoadPicture;
     inherited;
end;

procedure TCLDBImage.WMCut(var Message: TMessage);
begin
     CutToClipboard;
end;

procedure TCLDBImage.WMCopy(var Message: TMessage);
begin
     CopyToClipboard;
end;

procedure TCLDBImage.WMPaste(var Message: TMessage);
begin
     PasteFromClipboard;
end;

procedure TCLDBImage.WMSize(var Message: TMessage);
begin
     inherited;
     Invalidate;
end;

end.
